==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/block/block_entity/CellularBlockEntity.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.block.block_entity;

import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2680;
import vazkii.botania.api.block.Bound;
import vazkii.botania.common.block.flower.generating.DandelifeonBlockEntity;

public class CellularBlockEntity extends BotaniaBlockEntity {
	private static final String TAG_GENERATION = "generation";
	private static final String TAG_TICKED = "ticked";
	private static final String TAG_FLOWER_X = "flowerX";
	private static final String TAG_FLOWER_Y = "flowerY";
	private static final String TAG_FLOWER_Z = "flowerZ";
	private static final String TAG_VALID_X = "validX";
	private static final String TAG_VALID_Y = "validY";
	private static final String TAG_VALID_Z = "validZ";

	private int generation;
	private boolean ticked;
	private class_2338 flowerCoords = Bound.UNBOUND_POS;
	private class_2338 validCoords = Bound.UNBOUND_POS;

	public CellularBlockEntity(class_2338 pos, class_2680 state) {
		super(BotaniaBlockEntities.CELL_BLOCK, pos, state);
	}

	public void setGeneration(DandelifeonBlockEntity flower, int gen) {
		generation = gen;
		if (!ticked) {
			flowerCoords = flower.getEffectivePos();
			validCoords = method_11016();
			ticked = true;
		} else if (!validCoords.equals(method_11016()) || !flowerCoords.equals(flower.getEffectivePos())) {
			field_11863.method_8650(field_11867, false);
		}
	}

	public boolean isSameFlower(DandelifeonBlockEntity flower) {
		return validCoords.equals(method_11016()) && flowerCoords.equals(flower.getEffectivePos());
	}

	public int getGeneration() {
		return generation;
	}

	@Override
	public void writePacketNBT(class_2487 cmp) {
		cmp.method_10569(TAG_GENERATION, generation);
		cmp.method_10556(TAG_TICKED, ticked);
		if (ticked) {
			cmp.method_10569(TAG_FLOWER_X, flowerCoords.method_10263());
			cmp.method_10569(TAG_FLOWER_Y, flowerCoords.method_10264());
			cmp.method_10569(TAG_FLOWER_Z, flowerCoords.method_10260());
			cmp.method_10569(TAG_VALID_X, validCoords.method_10263());
			cmp.method_10569(TAG_VALID_Y, validCoords.method_10264());
			cmp.method_10569(TAG_VALID_Z, validCoords.method_10260());
		}
	}

	@Override
	public void readPacketNBT(class_2487 cmp) {
		generation = cmp.method_10550(TAG_GENERATION);
		ticked = cmp.method_10577(TAG_TICKED);
		if (ticked) {
			flowerCoords = new class_2338(
					cmp.method_10550(TAG_FLOWER_X),
					cmp.method_10550(TAG_FLOWER_Y),
					cmp.method_10550(TAG_FLOWER_Z)
			);
			validCoords = new class_2338(
					cmp.method_10550(TAG_VALID_X),
					cmp.method_10550(TAG_VALID_Y),
					cmp.method_10550(TAG_VALID_Z)
			);
		}
	}

}

==> Botania-1.20.1-451-FABRIC-20251204.072140-31-sources.jar: vazkii/botania/common/block/block_entity/CellularBlockEntity.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.block.block_entity;

import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2680;
import vazkii.botania.api.block.Bound;
import vazkii.botania.common.block.BotaniaBlocks;
import vazkii.botania.common.block.flower.generating.DandelifeonBlockEntity;
import vazkii.botania.common.block.flower.generating.DandelifeonBlockEntity.Cell;

public class CellularBlockEntity extends BotaniaBlockEntity {
	private static final String TAG_GENERATION = "generation";
	private static final String TAG_TICKED = "ticked";
	private static final String TAG_FLOWER_X = "flowerX";
	private static final String TAG_FLOWER_Y = "flowerY";
	private static final String TAG_FLOWER_Z = "flowerZ";
	private static final String TAG_VALID_X = "validX";
	private static final String TAG_VALID_Y = "validY";
	private static final String TAG_VALID_Z = "validZ";

	private int generation;
	private int nextGeneration;
	private boolean ticked;
	private class_2338 flowerCoords = Bound.UNBOUND_POS;
	private class_2338 validCoords = Bound.UNBOUND_POS;

	public CellularBlockEntity(class_2338 pos, class_2680 state) {
		super(BotaniaBlockEntities.CELL_BLOCK, pos, state);
	}

	public void setGeneration(int gen) {
		generation = gen;
	}

	public void setNextGeneration(DandelifeonBlockEntity flower, int gen) {
		nextGeneration = gen;
		method_10997().method_39279(method_11016(), BotaniaBlocks.cellBlock, 1);
		if (!ticked) {
			claim(flower);
			ticked = true;
		} else if (!validCoords.equals(method_11016()) || !flowerCoords.equals(flower.getEffectivePos())) {
			field_11863.method_8650(field_11867, false);
		}
	}

	public void claim(DandelifeonBlockEntity flower) {
		if (!ticked) {
			flowerCoords = flower.getEffectivePos();
			validCoords = method_11016();
		}
	}

	public void update(class_1937 level) {
		if (nextGeneration == Cell.DEAD) {
			level.method_8650(method_11016(), false);
		}
		generation = nextGeneration;
	}

	public boolean hasActiveParent(DandelifeonBlockEntity dandie) {
		return flowerCoords != null && dandie.method_10997().method_8321(flowerCoords) instanceof DandelifeonBlockEntity parent && dandie.method_10997().method_49803(flowerCoords) && (!dandie.overgrowthBoost || parent.isOnSpecialSoil());
	}

	public int getGeneration() {
		return generation;
	}

	@Override
	public void writePacketNBT(class_2487 cmp) {
		cmp.method_10569(TAG_GENERATION, generation);
		cmp.method_10556(TAG_TICKED, ticked);
		if (ticked) {
			cmp.method_10569(TAG_FLOWER_X, flowerCoords.method_10263());
			cmp.method_10569(TAG_FLOWER_Y, flowerCoords.method_10264());
			cmp.method_10569(TAG_FLOWER_Z, flowerCoords.method_10260());
			cmp.method_10569(TAG_VALID_X, validCoords.method_10263());
			cmp.method_10569(TAG_VALID_Y, validCoords.method_10264());
			cmp.method_10569(TAG_VALID_Z, validCoords.method_10260());
		}
	}

	@Override
	public void readPacketNBT(class_2487 cmp) {
		generation = cmp.method_10550(TAG_GENERATION);
		ticked = cmp.method_10577(TAG_TICKED);
		if (ticked) {
			flowerCoords = new class_2338(
					cmp.method_10550(TAG_FLOWER_X),
					cmp.method_10550(TAG_FLOWER_Y),
					cmp.method_10550(TAG_FLOWER_Z)
			);
			validCoords = new class_2338(
					cmp.method_10550(TAG_VALID_X),
					cmp.method_10550(TAG_VALID_Y),
					cmp.method_10550(TAG_VALID_Z)
			);
		}
	}

	@Override
	public boolean method_11011() {
		// targeting Create here, sorry about any instances of https://xkcd.com/1172/
		return true;
	}
}
